package cPanel::APIClient::X::API;

# Copyright 2020 cPanel, L. L. C.
# All rights reserved.
# http://cpanel.net
#
# This is free software; you can redistribute it and/or modify it under the
# same terms as Perl itself. See L<perlartistic>.

use strict;
use warnings;

=encoding utf-8

=head1 NAME

cPanel::APIClient::X::API

=head1 DESCRIPTION

This class indicates some failure on the API’s part to run the
requested function.

=cut

#----------------------------------------------------------------------

use parent qw( cPanel::APIClient::X::Base );

1;
